# -*- coding: utf-8 -*-
"""Script for auto-responding to a received SMS with data from a CSV.
"""

__author__ = "Kamran Riaz Khan"
__email__ = "krkhan@inspirated.com"
__version__ = "0.1"
__copyright__ = "Copyright (c) 2009 Kamran Riaz Khan"
__license__ = "Python"
__status__ = "Production"

import os
import appuifw
import contacts
import e32
import inbox
import logs
import messaging
import DSV

def exit_key_handler():
	app_lock.signal()

def message_received(id, keyword, db, content):
	box = inbox.Inbox()
	sms_text = box.content(id).strip()

	if sms_text.lower().find(keyword.lower()) == 0:
		sender = box.address(id)

		if sender[0] != '+' or sender[1:].isdigit() == False:
			content.add(u"Named contact encountered: %s\n" % sender)

			# Try to find the sender's number from log.
			logged_sms = logs.sms(mode = 'in')[0]

			# To confirm that the log entry is the same one we're concerned
			# about, do some checks.
			if logged_sms['subject'] == sms_text[:64]:
				number = logged_sms['number']
				contacts_db = contacts.open()
				found_contact = contacts_db.find(number)
				if found_contact:
					title_match = True
					for token in sender.split():
						if found_contact[0].title.find(token) == -1:
							title_match = False
					if title_match:
						sender = number

		primary = sms_text[len(keyword) + 1:]
		result = u"Received query %s from %s\n" % (primary, sender)
		if db.has_key(primary):
			result = result + u"Found record in database:\n"
			for (k, v) in db[primary].items():
				result = result + u"> %s: %s\n" % (k, v)
		else:
			result = result + u"Could not find record in database\n"
		result = result + "\n"

		content.add(result)
		sms_sent = False
		while sms_sent == False:
			try:
				messaging.sms_send(sender, result)
				sms_sent = True
			except RuntimeError:
				pass

		try:
			logfile = open(u"E:\\Others\\csvautoresponder.txt", "a")
			try:
				logfile.write(result)
			finally:
				logfile.close()
		except IOError:
			content.add(u"Could not write to log file")

if __name__ == "__main__":
	appuifw.app.title = u"CSV Auto Responder"

	csvdir = u"E:\\"
	selected_file = u""
	selected_path = csvdir + selected_file
	while os.path.isfile(selected_path) == False:
		if os.path.isdir(selected_path):
			csvdir = csvdir + selected_file

		files = map(unicode, os.listdir(csvdir))
		index = appuifw.selection_list(files)
		if index == None:
			break
		else:
			selected_file = files[index]

		if csvdir[-1] != "\\":
				csvdir = csvdir + "\\"

		selected_path = csvdir + selected_file

	content = appuifw.Text()
	appuifw.app.body = content

	if os.path.isfile(selected_path) == False:
		content.add(u"No file selected\n")
	else:
		content.add(u"File selected: %s\n" % selected_path)

	try:
		fsock = open(selected_path, "r")
		try:
			data = fsock.read()
			data = DSV.organizeIntoLines(data)
			data = DSV.importDSV(data, delimiter = ',')
		finally:
			fsock.close()

			header = data[0]
			db = {}
			for row in data[1:]:
				primary = row[0]
				db[primary] = {}
				for (k, v) in zip(header[1:], row[1:]):
					db[primary][k] = v

			keyword = appuifw.query(u"Enter Keyword:", 'text')

			box = inbox.Inbox()
			box.bind(lambda id: message_received(id, keyword, db, content))

			content.add(u"Waiting for SMS in the following format:\n")
			content.add(u"%s <%s>\n\n" % (keyword, header[0]))
	except IOError:
		content.add(u"Could not read %s\n" % selected_path)

	appuifw.app.exit_key_handler = exit_key_handler

	app_lock = e32.Ao_lock()
	app_lock.wait()

